<?php
require_once("cwfUI.inc.php");

class DbAdminProfileUI extends cwfUI {
  protected static $accessRight = "";
  protected static $dataClass = "DbAdmin";
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  public function __construct() {
    parent::__construct();
    $this->initialize();
  }
  
  protected function initialize() {
    parent::initialize();
    $this->data->fromArray($_SESSION['admin']);
    $this->title = _("Profile");
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // security
  static public function checkPrivileges($user, $action) {
    if($err = parent::checkPrivileges($user, $action))
      return $err;
    if(in_array($action, array("changepw")) &&
       (!isset($_POST["db_admin_id"]) || $_POST["db_admin_id"]!=$_SESSION['admin']['db_admin_id']))
      return _("You are not allowed to change another admins password.");
    
    return;
  }
  
  static public function isAllowedAction($action) {
    return in_array($action, array("changepw"));
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // handle data
  static public function getValidationRules() {
    $rules = array(
        "passwd"=>array("required"=>true),
        "passwd_repeat"=>array("equalTo"=>"#passwd")
      );
    $messages = array(
        "passwd"=>array("required"=>_("Please enter a password")),
        "passwd_repeat"=>array("equalTo"=>_("Passwords do not match."))
      );
    
    $validationRules = new FormValidation($rules, $messages);
    return $validationRules;
  }
  
  public function doAction(DbConnection $DB, $action) {
    $id = "";
    switch ($action) {
      case "changepw":
        $this->saveForm($DB);
        $id = $this->getId();
        break;
    }
    return $id;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // Database operations
  public function saveForm(DbConnection $DB) {
    if($this->data->fromArray($_POST)) {
      list($errno, $error) = $this->data->changePW($DB);
      if ($errno) {
        $this->error = $error;
        return 0;
      }
      return 1;
    }
    $this->error = _("Validation failed.");
    return 0;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // output
  public function getHTML(DbConnection $DB) {
?>
<form id="kdb_form" action="" method="post" enctype="multipart/form-data">
<div class="form_controls_top">
<p>
  <input type="submit" id="kdb_save" name="kdb_save" value="<?=_("Save")?>" onClick="kdbSaveEntry('DbAdminProfileUI', 'changepw'); return false;">
</p>
</div>

<h3 class="first"><?=_("Admin profile")?></h3>

<input type="hidden" id="db_admin_id" name="db_admin_id" class="id" value="<?= $this->data->db_admin_id ?>">

<p>
  <label for="login"><?=_("User name")?>:</label>
  <input type="text" id="login" name="login" value="<?=$this->data->login?>" disabled>
</p>

<p>
  <label for="email"><?=_("Email address")?>:</label>
  <input type="text" id="email" name="email" value="<?=$this->data->email?>" disabled>
</p>

<p>
  <label for="passwd"><?=_("Password")?>:</label>
  <input type="password" id="passwd" name="passwd" value="">
</p>

<p>
  <label for="passwd_repeat"><?=_("Repeat password")?>:</label>
  <input type="password" id="passwd_repeat" name="passwd_repeat" value="">
</p>

<p>
  <label for="ip_addresses"><?=_("IP address(es)")?>:</label>
  <input type="text" id="ip_addresses" name="ip_addresses" value="<?=$this->data->ip_addresses?>" disabled>
    <span></span>
</p>

<p>
  <label for="db_admin_group_ids"><?=_("Groups")?>:</label>
  <select id="db_admin_group_ids" name="db_admin_group_ids[]" multiple size="5" disabled>
<?php
    $groups = DbAdminGroup::readList($DB);
    foreach ($groups as $group)
      echo "    <option value={$group['id']}>{$group['name']}</option>\n";
?>
  </select>
</p>

<h3><?=_("Access rights")?></h3>
<div class="toggle disabled" id="access_rights">
  <div class="toggle_part toggle_available">
    <div><?=_("Actions")?>:</div>
    <ul>
<?php
    $accessCounter = 0;
    $accessNames = array();
    foreach (DbAdmin::getAccessRights() as $name => $text) {
      if (!in_array($name, $this->data->access_rights))
        echo "      <li id=\"{$name}\" name=\"toggle_{$accessCounter}\">{$text}<input type=\"hidden\" name=\"access_rights[]\" value=\"{$name}\" disabled></li>\n";
      else
        $accessNames[$accessCounter] = $name;
      $accessCounter++;
    }
?>
    </ul>
  </div>
  <div class="toggle_control"><span class="toggle_add">&raquo;</span><br><span class="toggle_remove">&laquo;</span></div>
  <div class="toggle_part toggle_selected toggle_sorted">
    <div><?=_("Selected actions")?>:</div>
    <ul>
<?php
    foreach ($accessNames as $accessCounter => $name) {
      echo "      <li id=\"{$name}\" name=\"toggle_{$accessCounter}\">".DbAdmin::getAccessRights($name)."<input type=\"hidden\" name=\"access_rights[]\" value=\"{$name}\"></li>\n";
    }
?>
    </ul>
  </div>
  <div class="toggle_control"><span class="toggle_up">&uArr;</span><br><span class="toggle_down">&dArr;</span></div>
</div>

<br style="clear:both;">

<div class="form_controls_bottom">
<p>
  <input type="submit" id="kdb_save2" name="kdb_save" value="<?=_("Save")?>" onClick="kdbSaveEntry('DbAdminProfileUI', 'changepw'); return false;">
</p>
</div>
</form>
<?php
  }
}

?>