<?php 
require_once("icwfData.inc.php");
require_once("FormValidation.inc.php");

class View implements icwfData {
  private static $noExport = array();
  private static $notInArray = array("validation");
  private $validation;
  
  private $view_id;
  private $view_name;
  private $columns;
  private $db_admin_ids;
  private $edit_booking_check;
  private $edit_user_data;
  private $where_clause;
  private $default_order_by;
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  private function initialize() {
    foreach ($this as $key => $value)
      if(in_array($key, array("db_admin_ids", "columns")))
        $this->$key = array();
      else
        $this->$key = null;
  }
  
  public function __construct(FormValidation $validation=null) {
    $this->initialize();
    
    $this->validation = $validation;
  }
    
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // get values
  public function __get($name) {
    if($name == "id") $name = "view_id";
    if (array_key_exists($name, get_class_vars(get_class())) && !in_array($name, self::$noExport))
      return $this->$name;
      
    return null;
  }

  public function getIDName() {
    return "view_id";
  }
  
  public function toArray() {
    $array = array();
    foreach ($this as $key => $value)
      if(!in_array($key, self::$notInArray))
        $array[$key] = $value; 
    return $array;
  }
  
  public function getPostBackData() {
    return array($this->getIDName()=>$this->id);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // set values
  public function setValidation(FormValidation $validation) {
    $this->validation = $validation;
  }

  public function __set($name, $value) {
    if($name == "id") $name = "view_id";
    if (array_key_exists($name, get_class_vars(get_class()))) {
      switch ($name) {
        case "validation":
          break;
          
        case "view_id":
        case "view_name":
        case "where_clause":
          $this->$name = $value;
          break;
        case "columns":
        case "db_admin_ids":
          if(is_array($value))
            $this->$name = $value;
          else
            $this->$name = array();
          break;

        default:
          $this->$name = ($value ? 1 : 0);
          break;
      }
    }
  }

  public function fromArray($array) {
    foreach ($this as $key => $value) {
      switch ($key) {
        case "validation":
          break;
          
        case "view_id":
        case "view_name":
        case "where_clause":
          if(isset($array[$key]))
            $this->$key = $array[$key];
          else
            $this->$key = null;
          break;
        case "columns":
        case "db_admin_ids":
          if(isset($array[$key]) && is_array($array[$key]))
            $this->$key = $array[$key];
          else
            $this->$key = array();
          break;

        default:
          $this->$key = ((isset($array[$key]) && $array[$key]) ? 1 : 0);
          break;
      }
    }
    return $this->validation->validate($array);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - read
  public function readData(DbConnection $DB, $conditions) {
    if(is_array($conditions)) {
      $whereConditions = array();
      foreach ($conditions as $key => $value)
        if(array_key_exists($key, get_class_vars(get_class())))
          $whereConditions[] = "$key=$value";
      
      $conditions = implode(" AND ", $whereConditions);
    }
    $result = $DB->query("SELECT * FROM views WHERE {$conditions};");
    
    if(!$DB->errno() && $DB->num_rows($result)>0) {
      $view = $DB->fetch_array($result);
      foreach ($view as $property => $value) {
        if(property_exists($this, $property)) {
          switch ($property) {
            case "columns":
            case "db_admin_ids":
              if($value)
                $this->$property = explode(",", $value);
              else
                $this->$property = array();
              break;
            default:
              $this->$property = $value;
              break;
          }
        }
      }
      
      return array(0, "");
    }
    else
      return array($DB->errno(), $DB->error());
  }
  
  static public function readList(DbConnection $DB, $filter="") {
    if($filter!="")
      $sql = "SELECT view_id id, view_name name FROM views WHERE {$filter} ORDER BY view_name;";
    else
      $sql = "SELECT view_id id, view_name name FROM views ORDER BY view_name;";
    $result = $DB->query($sql);
    
    $list = array();
    while ($view = $DB->fetch_array($result))
      $list[] = $view;
      
    return $list;
  }
  
  static public function getMenuArray() {
    global $DB;

    $menuArray=array();
    if(isset($_SESSION['admin'])) {
      $views = static::readList($DB, ($_SESSION['admin']['login']=='root' ? "" : "db_admin_ids RLIKE '(^$)|((^|,){$_SESSION['admin']['db_admin_id']}(,|$))'"));
      foreach ($views as $view)
        $menuArray["view_{$view['id']}"] = array('title'=>$view['name'], 'url'=>'index.php/view?view_id='.$view['id'], 'permission'=>'', 'submenu'=>'');
    }
    
    return $menuArray;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - write
  public function writeData(DbConnection $DB) {
    if(!isset($this->view_name) || !$this->view_name) return;
    if(isset($this->view_id) && $this->view_id) {
      $result = $DB->query("UPDATE views SET
          view_name='{$this->view_name}',
          columns='".implode(",", $this->columns)."',
          db_admin_ids='".implode(",", $this->db_admin_ids)."',
          edit_booking_check='{$this->edit_booking_check}',
          edit_user_data='{$this->edit_user_data}'
        WHERE view_id='{$this->view_id}';");
    }
    else {
      $result = $DB->query("INSERT INTO views(
          view_name,
          columns,
          db_admin_ids,
          edit_booking_check,
          edit_user_data
        ) VALUES(
          '{$this->view_name}',
          '".implode(",", $this->columns)."',
          '".implode(",", $this->db_admin_ids)."',
          '{$this->edit_booking_check}',
          '{$this->edit_user_data}'
        );");
      
      $this->view_id = $DB->insert_id();
    }
    return array($DB->errno(), $DB->error());
  }

  static public function deleteData(DbConnection $DB, $id) {
    if(!preg_match("~^[0-9]+$~", $id))
      return array(1, "ID not valid.");
      
    $result = $DB->query("DELETE FROM views WHERE view_id='{$id}';");
    
    if(!$DB->errno() && $DB->affected_rows()>0)
      return array(0, "");
    else 
      return array($DB->errno(), $DB->error());
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - misc
  static public function createTable(DbConnection $DB) {
    $result = $DB->query("
      CREATE TABLE views (
        view_id int unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY,
        view_name text NOT NULL,
        columns text,
        db_admin_ids text,
        edit_booking_check tinyint,
        edit_user_data tinyint
      );");
    if ($result) { 
      echo _("&nbsp;&nbsp;- creating table").": 'views'.<br>\n";
    }
    else { echo _("&nbsp;&nbsp;<b>- Error when creating table</b>")." 'views'<br>\n"; $error = 1; }
  }  
}
?>